// $Id:$
//
// Copyright (c) 2006
//      The Regents of the University of California.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that: (1) source code distributions
// retain the above copyright notice and this paragraph in its entirety, (2)
// distributions including binary code include the above copyright notice and
// this paragraph in its entirety in the documentation or other materials
// provided with the distribution, and (3) all advertising materials mentioning
// features or use of this software display the following acknowledgement:
// ``This product includes software developed by the University of California,
// Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
// the University nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#ifndef rewriter_h
#define rewriter_h

class TracePacket;

class Rewriter {
public:
	virtual ~Rewriter()	{};

	virtual void Done()	{};

	virtual void WriteData(int is_orig, int len, const u_char* data) = 0;
	virtual void WriteData(int is_orig, const char* data) = 0;
	virtual void WriteData(int is_orig, int len, const char* data) = 0;
	virtual void WriteData(int is_orig, const BroString* str) = 0;

	virtual void Push(int is_orig) = 0;

	virtual void AbortPackets(int apply_to_future) = 0;
	virtual void CommitPackets(int apply_to_future) = 0;

	virtual unsigned int ReserveSlot() = 0;
	virtual int SeekSlot(unsigned int slot) = 0;
	virtual int ReturnFromSlot() = 0;
	virtual int ReleaseSlot(unsigned int slot) = 0;

	// Needed by all rewriters.
	virtual TracePacket* CurrentPacket() const = 0;
	virtual TracePacket* RewritePacket() const = 0;

	// Whether to not anonymize client/server IP addresses.
	virtual int LeaveAddrInTheClear(int is_orig) = 0;
};

extern Rewriter* get_trace_rewriter(Val* conn_val);
extern Rewriter* get_trace_rewriter(Connection* conn);

// This is the actual packet.
class TracePacket {
public:
	virtual ~TracePacket()	{ }

	virtual RecordVal* PacketVal() = 0;
	virtual const double TimeStamp() const = 0;
};

#endif
